function [Up_az,Up_90,Up_az_el,number_alias,VD_unfolded_final,VD_unf_flag,percentage_VD_unf_flag]=dealiasing(path_processed_data,azimuth,range_cc,ele,ele_cc,wind_dir,wind_speed,velo,nyquist_velo,dbz)

%*Function* dealiasing.m
% @version sacr 1.1

% *SRC_FILE*
% sacrm/dealiasing.m
  

%%%%%%%%%%%%%%%%%%%% DESCRIPTION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function de-aliases/unfolds doppler velocity 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% INPUTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% azimuth      : scan fixed azimuth angle (degrees)
%               one value repeated [number of elevation angles,1]
% range_cc     : range corrected for Earth's curvature (m)
%                [number of elevation angles,number of range gates]
% ele_cc       : elevation corrected for Earth's curvature (degrees)
%                [number of elevation angles,number of range gates]
% ele          : elevation angles (degrees) [number of elevation angles,1]
% wind_dir     : wind direction from sounding (degrees) 
%                [number of elevation angles,number of range gates]
% wind_speed   : wind speed from sounding (m/s)
%                [number of elevation angles,number of range gates]
% velo         : doppler velocity (m/s)
%                [number of elevation angles,number of range gates]
% nyquist_velo : nyquist velocity (m/s) [1]
% dbz          : reflectivity (dBZ)
%                [number of elevation angles,number of range gates]                                              
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% OUTPUTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Up_az             : wind speed projected to the azimuth scan (m/s)
%                     [number of elevation angles,number of range gates]
% Up_90             : wind speed projected 90 degrees to the scan (m/s)
%                     [number of elevation angles,number of range gates]
% Up_az_el          : wind speed projected to the azimuth and elevation of
%                     the scan (m/s)
%                     [number of elevation angles,number of range gates]
% number_alias      : number of time the nyquist velocity was added to the
%                     velocity 
%                     [number of elevation angles,number of range gates]
% VD_unfolded_final : velocity unfolded (m/s)
%                     [number of elevation angles,number of range gates]
% VD_unfolded_flag  : identifies elevations where the unfolded failed
%                     [number of elevation angles,1]
% velo_flag         : identifies elevations that have been unfolded
%                     [number of elevation angles,1]
% percentage_VD_unfolded_flag : percentage of elevation angles where
%                      unfolding failed [1]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  This program  contains the subroutines needed for procesing raw sacr data
%
%
%  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
%  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca,
%  iengjo@meteo.mcgill.ca
%
%  This source code is free software: you can redistribute it and/or modify
%  it under the terms of the GNU Lesser General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU Lesser General Public License for more details.
%
%  You should have received a copy of the GNU Lesser General Public License
%  along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% ----------------------------------------------------------------------------

E = length(ele);

%%%% Project wind speed from sounding data on different planes %%%%

d_azimuth  = repmat(azimuth,1,size(range_cc,2))-wind_dir;  %corrected
d_azimuth1 = (repmat(azimuth,1,size(range_cc,2))+90)-wind_dir; %corrected
Up_az      = wind_speed.*cos(d_azimuth*pi/180);
Up_az_el   = Up_az.*cos(ele_cc*pi/180);
Up_90      = wind_speed.*cos(d_azimuth1*pi/180);    
  
%%% Calculate number of aliasing based on wind projection on the %%
%%% scanning plane %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

number_alias = round((Up_az_el-velo)/(2*nyquist_velo)); 

%%%%%%%%% Initial de_aliasing %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

VD_unfolded = velo+number_alias*2*nyquist_velo; 

%%%%%%%%% Doppler velocity quality mask based on reflectivity %%%%

dbz_doppler_threshold = -40+20*log10(range_cc/1000);
k = find(dbz<dbz_doppler_threshold);

%%%%% Bottom Up smoothing using pixels with high reflectivity %%%%

VD_unfolded_BUsmooting    = VD_unfolded;
VD_unfolded_BUsmooting(k) = NaN;

for e = 1:E;
    cloud_pix=find(isnan(VD_unfolded_BUsmooting(e,:))==0); % find cloud pixels
        if length(cloud_pix)>1                                 % if there are cloud pixels
            diff_VD = diff(VD_unfolded_BUsmooting(e,cloud_pix));    
                if max(abs(diff_VD)) > nyquist_velo             % doesn;t do this.... find if there are two adjacent cloud pixels that have velocities difference above the nyquist (indicator of bad unfolding)
                    for lk = 1:length(cloud_pix)-1;                        % loop through the cloud pixels and ajust the pixel above to match the pixel below with a difference smaller than the nyquist velocity
                        delta_v = VD_unfolded_BUsmooting(e,cloud_pix(lk))-VD_unfolded_BUsmooting(e,cloud_pix(lk+1));
                            if delta_v > nyquist_velo 
                                VD_unfolded_BUsmooting(e,cloud_pix(lk+1)) = VD_unfolded(e,cloud_pix(lk+1))+2*nyquist_velo;
                            elseif delta_v < -nyquist_velo 
                                VD_unfolded_BUsmooting(e,cloud_pix(lk+1)) = VD_unfolded(e,cloud_pix(lk+1))-2*nyquist_velo;
                            end
                    end
                end
        end
end

cloud_pix = []; diff_VD = []; lk = [];

%%%%% Top Down smoothing using pixels with high reflectivity %%%%

VD_unfolded_TDpass    = VD_unfolded;
VD_unfolded_TDpass(k) = NaN;

for e = 1:E;
    cloud_pix=find(isnan(VD_unfolded_TDpass(e,:))==0);   % find cloud pixels
        if length(cloud_pix)>1                               % if there are cloud pixels
            diff_VD = diff(VD_unfolded_TDpass(e,cloud_pix));        
                if max(abs(diff_VD)) > nyquist_velo           % find if there are two adjacent cloud pixels that have velocities difference above the nyquist (indicator of bad unfolding)
                    for lk = length(cloud_pix):-1:2;                     % loop through the cloud pixels and ajust the pixel below to match the pixel above with a difference smaller than the nyquist velocity
                      delta_v = VD_unfolded_TDpass(e,cloud_pix(lk))-VD_unfolded_TDpass(e,cloud_pix(lk-1));
                         if delta_v > nyquist_velo
           
                            VD_unfolded_TDpass(e,cloud_pix(lk-1)) = VD_unfolded(e,cloud_pix(lk-1))+2*nyquist_velo;
                         elseif delta_v < -nyquist_velo
    
                            VD_unfolded_TDpass(e,cloud_pix(lk-1)) = VD_unfolded(e,cloud_pix(lk-1))-2*nyquist_velo;
                         end
                    end
                end
        end
end

cloud_pix = []; diff_VD = []; lk = [];

%%%% Replace the cloud pixels with reflectivities below %%%%%%%%
%%%% the threshold by the initial unfolding values %%%%%%%%%%%%%
 
VD_unfolded_BUsmooting(k) = VD_unfolded(k);
VD_unfolded_TDpass(k)     = VD_unfolded(k);

%%%% Choose between the original unfolding and both smoothing %%
%%%% unsing a confidence test %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

VD_unfolded_final = VD_unfolded;

for e11 = 1:E;
cost_function_In(e11) = nanstd(Up_az_el(e11,:)-VD_unfolded(e11,:));
cost_function_BU(e11) = nanstd(Up_az_el(e11,:)-VD_unfolded_BUsmooting(e11,:));
cost_function_TD(e11) = nanstd(Up_az_el(e11,:)-VD_unfolded_TDpass(e11,:));

end

for e12 = 1:E;
    if cost_function_BU(e12) > cost_function_TD(e12)
        if cost_function_In(e12) > cost_function_TD(e12)
        VD_unfolded_final(e12,:) = VD_unfolded_TDpass(e12,:);
        end
    else 
        if cost_function_In(e12) > cost_function_BU(e12)
        VD_unfolded_final(e12,:) = VD_unfolded_BUsmooting(e12,:);
        end
    end
end

%%%%%%% Create flags to check dealiasing quality %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%


VD_unf_flag = zeros([length(ele),1]);
 

for e=1:length(ele)

flag  = find(abs(diff(VD_unfolded(e,:)))>nyquist_velo);
flag(flag<=10)=NaN;flag(flag>=(size(range_cc,2)-10))=NaN;
 
    if numel(flag(isnan(flag)==0)) >0 ;
        VD_unf_flag(e)=1;
    end
 
end

percentage_VD_unf_flag=100*(sum(VD_unf_flag)/length(ele));
 

cd (path_processed_data); 
end % END function
